integer FUNCTION ifsnow(t2m,p,rh,g0,st,ol)
! determine whether it snowing or raining condition
! INPUTS:
! t2m: 2m temperature in C - must have valid input value
! p: surface pressure in mb
! rh: relative humidity at 2m in %, -999.9 if unavailable
! g0: lapse rate of temperature of the lowest 0.5 km, in unit C/km 
!     g0 is positive when temperature decreases with height
!     -999.9 if not available
! st: surface skin temperature in C, -999.9 if not available
! ol: ocean or land, 0=ocean, else=land, must have valid input value
!
! OUTPUTS:
!-1: indeterminable
! 0: Snowfall with less than 50% probability
! 1: Snowfall with over 50%, but less than 75% probability
! 2: Snowfall with over 75%, but less than 90% probability
! 3: Snowfall with over 90% probability 

implicit none
integer :: ol
real :: t2m,rh,g0,st,p
real :: probsnow,wk

if(t2m .le. -999 .or. t2m .ge. 999) then
ifsnow = -1
return
endif
wk = probsnow(t2m,p,rh,g0,st,ol)
if(wk .ge. 0.9) then
 ifsnow=3
else if(wk .ge. 0.75) then
 ifsnow=2
else if(wk .ge. 0.5) then
 ifsnow=1
else
 ifsnow=0
endif
return
end

real function probsnow(t2m,p,rh,g0,st0,ol)

use WaterVapor
implicit none

real :: t2m,p,rh,g0,st0
integer :: ol
integer,save :: iost10(10)=0
!character (len=80) :: problutdir
include "cntdir.inc"
! *************************** Input data ***************************
! Enter -999.9 or smaller for unavailable data (p, rh, g0, st)
! t2m: Must have valid input value
! ol: Must have valid input value
!real :: t2m		! 2-m temperature (deg C)
!real :: p		! Surface pressure (hPa)
!real :: rh		! Relative humidity (%)
!real :: g0		! Lapse rate, surface to 0.5 km (deg C / km)
!real :: st0		! Surface skin temperature (deg C)
!integer :: ol		! Surface type: ocean = 0, land = 1(default)
! ******************************************************************

real,save :: OCEANcpssltw(5,5,3), OCEANcpsltw(15,10), OCEANcpsstw(15,3), OCEANcpstw(20), OCEANcpst(20) 
real,save :: LANDcpssltw(15,10,4), LANDcpsltw(15,10), LANDcpsstw(15,4), LANDcpstw(20), LANDcpst(20)
real :: tvals20(20), tvals15(15), tvals5(5), skto3(3), sktl4(4), lr10(10), lr5(5)
real :: tw, st, res
integer :: i, j, sza, szb, szc

tvals20 = (/ (i-9.5, i = 0,19) /)
tvals15 = (/ (i-7, i = 0,14) /)
tvals5 = (/ ((i-2)*3, i = 0,4) /)
skto3 = (/ ((i*10)+270, i = 0,2) /)
sktl4 = (/ ((i*10)+260, i = 0,3) /)
lr10 = (/ ((i*2)-6.5, i = 0,9) /)
lr5 = (/ ((i*4)-5.5, i = 0,4) /)

! treat as land if ol is not set to be zero
if(ol.eq.0) then
if (p.gt.-999 .and. rh.gt.-999 .and. g0.gt.-999 .and. st0.gt.-999) goto 1000
if (p.gt.-999 .and. rh.gt.-999 .and. g0.gt.-999 .and. st0.le.-999) goto 1001
if (p.gt.-999 .and. rh.gt.-999 .and. g0.le.-999 .and. st0.gt.-999) goto 1002
if (p.gt.-999 .and. rh.gt.-999 .and. g0.le.-999 .and. st0.le.-999) goto 1003
if (p.le.-999 .or. rh.le.-999) goto 1004
else 
if (p.gt.-999 .and. rh.gt.-999 .and. g0.gt.-999 .and. st0.gt.-999) goto 1005
if (p.gt.-999 .and. rh.gt.-999 .and. g0.gt.-999 .and. st0.le.-999) goto 1006
if (p.gt.-999 .and. rh.gt.-999 .and. g0.le.-999 .and. st0.gt.-999) goto 1007
if (p.gt.-999 .and. rh.gt.-999 .and. g0.le.-999 .and. st0.le.-999) goto 1008
if (p.le.-999 .or. rh.le.-999) goto 1009
endif

1000 continue   ! ocean all parameters available

tw=rh2tw(p,t2m,rh)
if(iost10(1).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/OCEANcpssltw.dat', &
     status = 'old', action = 'read')
do i = 1, 5
  do j = 1, 5
    read(100,*) OCEANcpssltw(i,j,:)
  end do
end do
close(100)
iost10(1)=1
endif

st = st0+273.15
sza = size(OCEANcpssltw, 1)
szb = size(OCEANcpssltw, 2)
szc = size(OCEANcpssltw, 3)
if (tw.lt.-6) then
  tw=-6
else if (tw.gt.6) then
  tw=6
end if
if (g0.lt.-5.5) then
  g0=-5.5
else if (g0.gt.10.5) then
  g0=10.5
end if
if (st.lt.270) then
  st=270
else if (st.gt.290) then
  st=290
end if

CALL getres3d(OCEANcpssltw,sza,szb,szc,tvals5,lr5,skto3,tw,g0,st,res)
goto 1010

1001 continue   ! ocean, no st
 
tw=rh2tw(p,t2m,rh)
if(iost10(2).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/OCEANcpsltw.dat', &
     status = 'old', action = 'read')
do i = 1, 15
  read(100,*) OCEANcpsltw(i,:)
end do
close(100)
iost10(2)=1
endif

sza = size(OCEANcpsltw, 1)
szb = size(OCEANcpsltw, 2)
if (tw.lt.-7) then
  tw=-7
else if (tw.gt.7) then
  tw=7
end if
if (g0.lt.-6.5) then
  g0=-6.5
else if (g0.gt.11.5) then
  g0=11.5
end if
CALL getres2d(OCEANcpsltw,sza,szb,tvals15,lr10,tw,g0,res)
goto 1010

1002 continue  ! ocean, no lapse rate 
tw=rh2tw(p,t2m,rh)
if(iost10(3).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/OCEANcpsstw.dat', &
     status = 'old', action = 'read')
do i = 1, 15
  read(100,*) OCEANcpsstw(i,:)
end do
close(100)
iost10(3)=1
endif

st = st0+273.15
sza = size(OCEANcpsstw, 1)
szb = size(OCEANcpsstw, 2)
if (tw.lt.-7) then
  tw=-7
else if (tw.gt.7) then
  tw=7
end if
if (st.lt.270) then
  st=270
else if (st.gt.290) then
  st=290
end if
CALL getres2d(OCEANcpsstw,sza,szb,tvals15,skto3,tw,st,res)
goto 1010

1003 continue  ! ocean, no st + no lapse_rate
tw=rh2tw(p,t2m,rh)
if(iost10(4).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/OCEANcpstw.dat', &
     status = 'old', action = 'read')
read(100,*) OCEANcpstw(:)
close(100)
iost10(4)=1
endif

sza=size(OCEANcpstw)
if (tw.lt.-9.5) then
  tw=-9.5
else if (tw.gt.9.5) then
  tw=9.5
end if
CALL getres1d(OCEANcpstw,sza,tvals20,tw,res)
goto 1010

1004  continue ! ocean, only t2m
if(iost10(5).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/OCEANcpst.dat', &
     status = 'old', action = 'read')
read(100,*) OCEANcpst(:)
close(100)
iost10(5)=1
endif

sza=size(OCEANcpst)
if (t2m.lt.-9.5) then
  t2m=-9.5
else if (t2m.gt.9.5) then
  t2m=9.5
end if
CALL getres1d(OCEANcpst,sza,tvals20,t2m,res)
goto 1010

!!! Land
1005 continue  ! land, all parameters available
tw=rh2tw(p,t2m,rh)
if(iost10(6).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/LANDcpssltw.dat', &
     status = 'old', action = 'read')
do i = 1, 15
  do j = 1, 10
    read(100,*) LANDcpssltw(i,j,:)
  end do
end do
close(100)
iost10(6)=1
endif

st = st0+273.15
sza = size(LANDcpssltw, 1)
szb = size(LANDcpssltw, 2)
szc = size(LANDcpssltw, 3)
if (tw.lt.-7) then
  tw=-7
else if (tw.gt.7) then
  tw=7
end if
if (g0.lt.-6.5) then
  g0=-6.5
else if (g0.gt.11.5) then
  g0=11.5
end if
if (st.lt.260) then
  st=260
else if (st.gt.290) then
  st=290
end if
CALL getres3d(LANDcpssltw,sza,szb,szc,tvals15,lr10,sktl4,tw,g0,st,res)
goto 1010

1006 continue  ! land no st
tw=rh2tw(p,t2m,rh)
if(iost10(7).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/LANDcpsltw.dat', &
     status = 'old', action = 'read')
do i = 1, 15
    read(100,*) LANDcpsltw(i,:)
end do
close(100)
iost10(7)=1
endif

sza = size(LANDcpsltw, 1)
szb = size(LANDcpsltw, 2)
if (tw.lt.-7) then
  tw=-7
else if (tw.gt.7) then
  tw=7
end if
if (g0.lt.-6.5) then
  g0=-6.5
else if (g0.gt.11.5) then
  g0=11.5
end if
CALL getres2d(LANDcpsltw,sza,szb,tvals15,lr10,tw,g0,res)
goto 1010

1007 continue ! land, no lapse_rate
tw=rh2tw(p,t2m,rh)
if(iost10(8).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/LANDcpsstw.dat', &
     status = 'old', action = 'read')
do i = 1, 15
    read(100,*) LANDcpsstw(i,:)
end do
close(100)
iost10(8)=1
endif
st = st0+273.15
sza = size(LANDcpsstw, 1)
szb = size(LANDcpsstw, 2)
if (tw.lt.-7) then
  tw=-7
else if (tw.gt.7) then
  tw=7
end if
if (st.lt.260) then
  st=260
else if (st.gt.290) then
  st=290
end if
CALL getres2d(LANDcpsstw,sza,szb,tvals15,sktl4,tw,st,res)
goto 1010

1008 continue ! land, no st + no lapse_rate
tw=rh2tw(p,t2m,rh)
if(iost10(9).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/LANDcpstw.dat', &
     status = 'old', action = 'read')
read(100,*) LANDcpstw(:)
close(100)
iost10(9)=1
endif

sza=size(LANDcpstw)
if (tw.lt.-9.5) then
  tw=-9.5
else if (tw.gt.9.5) then
  tw=9.5
end if
CALL getres1d(LANDcpstw,sza,tvals20,tw,res)
goto 1010

1009 continue ! only t2m
if(iost10(10).eq.0) then
open(unit = 100, file = trim(problutdir)//'/LUTs/LANDcpst.dat', &
     status = 'old', action = 'read')
read(100,*) LANDcpst(:)
close(100)
iost10(10)=1
endif

sza=size(LANDcpst)
if (t2m.lt.-9.5) then
  t2m=-9.5
else if (t2m.gt.9.5) then
  t2m=9.5
end if
CALL getres1d(LANDcpst,sza,tvals20,t2m,res)

1010 probsnow = res
return
end
